#ifndef GAME_HPP
#define GAME_HPP

#include "AbstractGame.hpp"
#include "History.hpp"
#include "HistoryLoader.hpp"
#include "HistorySaver.hpp"
#include "Controlers/Controler.hpp"

#include "Utils/Notifier.hpp"
#include "Utils/Listener.hpp"
#include "Renderers/Renderer.hpp"

#include <vector>
#include <iostream>
#include <cassert>

template <typename GameBoard,typename GameRules/*, class ClockType*/>
class Game : public AbstractGame
{
    typedef typename GameBoard::Coords Coords;

private:

    GameBoard m_board;
    GameRules m_rules;
//        ClockType m_clocks[GameRules::NB_PLAYERS];

    std::vector<Controler<GameBoard>*> m_controlers;

    Notifier<Move<Coords> > m_gameNotifier;
    Notifier<Move<Coords> > m_statusNotifier;

    History<Move<Coords> > m_history;

    int moves;      /*!< Moves counter for the game (used to determine which should player play)*/

    int m_victoriousColor;      /*!< player color who wins the game ; -1 if no one wins (not yet) */
    bool m_draw;                /*!< true if the game ended in a draw */

    bool m_inited;          /*!< true if the game is ready to play */

    /*! \brief plays a \a move
     *  Checks the move, and apply it to the board if it is legal
     *  The listeners are prevented about the new move
     *  \param
     */
    void playMove(Move<Coords>& move)
    {
        int color = getCurrentColorTurn();

        std::cout << "Legal move : " << move.isLegal();
        if ( move.isLegal() && m_rules.check(m_board,move) )
        {
            std::cout << "Move accepted" << std::endl;

            m_rules.move(m_board,move);
            m_board.move(move);

            m_gameNotifier.notify(move);

            m_history.add(move);
            m_draw = m_rules.checkDraw(m_history);
            if ( m_rules.checkVictory(m_board,color) )
            {
                m_victoriousColor = color;
            }

            moves++;
        }
        else
        {
            move.invalidate();
        }

        m_statusNotifier.notify(move);
    }

    /*! \brief Init the game, if it was not done
     *  The game initiation consist to check if enough player are present
     *  and to init the board.
     *  An additional move is send to observers just to force a redisplay
     *  \return false if no enough players are present
     */
    bool init()
    {
        if ( m_inited == false )
        {
            if ( m_controlers.size() != GameRules::NB_PLAYERS)
            {
                // Not enough players
                return false;
            }

            m_rules.init(m_board);
            // Dummy move to display the game
            m_statusNotifier.notify(Move<typename GameBoard::Coords>());
        }

        m_inited = true;

        return true;
    }

public:

    /*!
     */
    Game()
        :moves(0),m_victoriousColor(-1),m_draw(false),m_inited(false)
    {
    }

    /*! The controlers are destructed */
    ~Game()
    {
        for ( typename std::vector<Controler<GameBoard>*>::const_iterator itCont = m_controlers.begin() ; itCont != m_controlers.end() ; ++itCont )
        {
            delete *itCont;
        }
    }

    /*! Gets the board
     *  \return
     */
    const GameBoard* getBoard() {return &m_board; }

    /*! Gets the rules
     *  \return
     */
    const GameRules* getRules() {return &m_rules; }

    /*! Adds a controler to give moves (aka: player)
     *  \param pControler
     *  \return false if enough controlers are in the game
     */
    bool addControler(Controler<GameBoard>* pControler)
    {
        assert(pControler);

        size_t nbControlers = m_controlers.size();
        // Add only if we did not reach the amount of required player
        if ( nbControlers < GameRules::NB_PLAYERS )
        {
            pControler->setColorControled(nbControlers); // The color is determined by the order o insertion
            m_controlers.push_back(pControler);
            m_gameNotifier.addListener(pControler);

            return true;
        }

        return false;
    }

    /*! Adds a listener for moves (aka: UI)
     *  \param pListener
     *  \return always true
     */
    bool addStatusListener(Listener<Move<Coords> >* pListener)
    {
        assert(pListener);

        m_statusNotifier.addListener(pListener);

        return true;
    }

    bool checkMove()
    {
        return m_rules.check(m_board);
    }

    /*!
     *  Plays the game until the \c rules declare that there is one player victorious
     *  or a draw is trigerred
     */
    bool play()
    {
        if ( init() == false )
        {
            return false;
        }

        do
        {
            int color = getCurrentColorTurn();
//                m_clocks[color].start();
            // GetTime
            Move<typename GameBoard::Coords> m = m_controlers[color]->getMove();
            playMove(m);

//                m_clocks[color].stop();
        }while(m_victoriousColor == -1 && m_draw == false);

/*
        std::cout << "Pat : " << m_draw << std::endl;
        std::cout << m_victoriousColor << " is checkmate" << std::endl;
*/
        return true;
    }

    /*! Return the color to which player to play
     *  \return
     */
    int getCurrentColorTurn()const
    {
        return (moves % GameRules::NB_PLAYERS);
    }

    int getVictoriousColor()const
    {
        return m_victoriousColor;
    }

    bool isVictory()const
    {
        return m_victoriousColor != -1;
    }

    bool isDraw()const
    {
        return m_draw;
    }

    bool loadHistory(const std::string& fileName)
    {
        bool error = false;

        m_history = HistoryLoader<Move<Coords> >::load(fileName,error);
        if ( error == false )
        {
            if ( init() == false )
            {
                return false;
            }

            size_t nbElem = m_history.size();
            for ( size_t i = 0 ; i < nbElem ; i++ )
            {
                std::pair<Move<ChessBoard::Coords>,Move<ChessBoard::Coords> > tdata = m_history.get(i);
                playMove(tdata.first);
                playMove(tdata.second);
            }
        }

        return error;
    }

    bool saveHistory(const std::string& fileName)
    {
        return HistorySaver<Move<Coords> >::save(fileName, m_history);
    }
};

/*! \class Game Game.hpp Game/Game.hpp
 *  \brief A game is just observers / controlers
 *
 *  Controlers are seen like players. They propose moves
 *  and the rules checks the correctness of these. If it is
 *  the board will apply the move.
 *  You can have observers (generally UI) to look at the game.
 *  These just knows the move accepted.
 *  The game will just play by rotating players (controlers to who
 *  we are asking for moves) and accepting the moves.
 *  The game will stop when a victory or a draw is reached
 */

#endif
