#ifndef HISTORY_HPP
#define HISTORY_HPP

#include <vector>
#include "Game/Move.hpp"
#include "Game/Boards/ChessBoard.hpp"

template <typename DataType>
class History
{
private:

    std::vector<DataType > m_history;   /*!< Data container */

public:

    /*! Adds data
     *  \param data to add in history
     */
    void add(const DataType& data);
};

template <>
class History<Move<ChessBoard::Coords> >
{
    typedef std::pair<Move<ChessBoard::Coords>,Move<ChessBoard::Coords> > TurnData;

private:

    std::vector< TurnData > m_history;  /*!< Data container */
    TurnData m_tmpData;                 /*!< Temporary container that will be pushed when the pair is completed */

public:


    /*! Adds data
     *  \param data to add in history
     *  \warning the data are not directly added to the history. It will be done only
     *  when the pair is completed
     */
    void add(const Move<ChessBoard::Coords>& data);

    /*! Get the number of data store
     *  \return the size of history
     */
    size_t size()const { return m_history.size(); }

    /*! Get the move number \c i for \c color
     *  \param i the number of the move to get
     *  \param color the move color to get
     *  \return the corresponding move
     */
    const Move<ChessBoard::Coords>& get(size_t i, int color)const;

    /*! Get the pair of move numbered \c i
     *  \param i the number of the pair to get
     *  \return the corresponding pair
     */
    const TurnData& get(size_t i)const;
};

/*! \class History History.hpp Game/History.hpp
 *  \brief Generic history
 *
 *  An history is simply a bank of data. The function add() injects a new data
 *  in the history
 */

/*! \class History<Move<ChessBoard::Coords> > History.hpp Game/History.hpp
 *  \brief Specific history for chess game moves
 *
 *  This history will keep moves by pair. Since the history can be considered as a table
 *  each line will contain the move for white player and the move for black player. TurnData
 *  is a shortcut to represents this pair. You can get a pair using get(size_t i), or a precise move for
 *  one of the player using get(size_t i, int color). The index passed to these functions
 *  should be in the range [0 - size()].
 *
 *  A move is not directly pushed in the history. This one has to be completed (after two effective call
 *  of add()) to see the move added to the history.
 *
 * \sa Move
 */


#endif
