#ifndef PIECE_HPP
#define PIECE_HPP

#include <string>
#include <map>

class Piece
{
private:
    bool m_valid;    /*!< true if the piece is valid (false would also mean no piece */

    int m_color;     /*!< Player owner of the piece */
    int m_type;      /*!< type */

    typedef std::map<std::string, int> PropertyMap;   /*!< Type for the additional information about the piece */
    PropertyMap m_properties;

public:
    /*! \brief Default constructor creating an invalid piece
     */
    Piece():m_valid(false) {}

    /*! \brief Constructor building a piece of type <a>type</a> for the owner <a>owner</a>
     *  \param owner the color of the owner of the piece
     *  \param type the type of the piece to create
     */
    Piece(int owner, int type);

    /*!
        \return true if the piece is valid (is false when it is not a piece)
     */
    bool isValid()const { return m_valid; }

    /*! \brief Sets a new property named <a>name</a> and assigned to the <a>value</a>
        \param name the name of the new property
        \param value the value associated to the property 
     */
    void setProperty(const std::string & name, int value);

    /*! \brief Checks if the named property <a>name</a> is available for this piece
        \param name the name of the property to check
     */
    bool hasProperty(const std::string& name)const;

   /*! \brief Gets the value associated with the property named <a>name</a>
       \param name the name of the property for which the value is returned
     */
    int getProperty(const std::string& name)const;

    /*! \return Gets the color of the piece
     */
    int color()const { return m_color; }

   /*! \return Gets the type of the piece
     */
    int type()const { return m_type; }
};

/*! \class Piece Piece.hpp Game/Piece.hpp
 *  \brief Represents a piece in the game
 *
 * The Piece does only contain the <a>type</a> (\c type()) and the <a>color</a> (\c color()) of the owner. Moreover, to be more generic,
 * additionnal information can be added using property (a value associated to a name). Easy to use function are provided
 * to interact with the properties : \c setProperty() \c hasProperty() and \c getProperty().
 * The piece can also be invalid, for example it is not a piece (no piece present on the board at this piece)
 */

namespace SPiece
{
    static const Piece InvalidPiece = Piece();
}

#endif
