#ifndef QT2DDISPLAY_HPP
#define QT2DDISPLAY_HPP

#include <QWidget>
#include <QGraphicsScene>
#include <QGraphicsView>

#include <QGraphicsRectItem>
#include <QGraphicsPixmapItem>

#include <QPropertyAnimation>

#include "Renderer2D.hpp"

#include "Game/Boards/ChessBoard.hpp"
#include "Game/Rules/ChessRules.hpp"
#include "Game/Move.hpp"
#include "Qt2DPiece.hpp"

#include "UI/PromotionDialog.hpp"


class Qt2DDisplay : public QGraphicsView, public Renderer2D<ChessBoard>
{
    Q_OBJECT

    typedef Move<ChessBoard::Coords> CMove;

private:

    QGraphicsScene m_scene;                     /*!< */
    std::vector<Qt2DPiece*> m_pieces;           /*!< */
    std::vector<QGraphicsRectItem*> m_helpers;  /*!< Items to display help */

    const ChessRules* m_pRules;                 /*!< rules used to display help */

    std::map<int, std::pair<QString,QString> > m_piecesPixmap;      /*!< */

    int m_caseWidth;        /*!< */
    int m_caseHeight;       /*!< */

    /*! \brief Builds the map for pixmap file names
     */
    void loadPieces();

    /*! \brief Create the board (tiles)
     */
    void createBoard();

    /*! \brief Destroy all the screen
     *  Removes the boards and the pieces
     */
    void clearScreen();

    /*! \brief Destroy the pieces
     */
    void clearPieces();

    /*! \brief a new board is available
     */
    void boardChanged()
    {
        createBoard();
    }

    /*! \brief signal to redraw the game
     */
    Q_SIGNAL void gameUpdateSignal();

    /*! \brief Redraw the game after an event
     */
    Q_SLOT void updateGame();

    /*! Wake up the QtControler for a new move
     */
    Q_SIGNAL void movePlayed(Move<ChessBoard::Coords>);

    /*! Emit the move from the notification to the rest of the UI
     *  \param
     */
    Q_SIGNAL void pieceMoved(CMove);

    /*! \brief Collects move from UI to send it to the controler
     *  \param start the start position in mouse coords
     *  \param end the end position in mouse coords
     *  \param color
     */
    Q_SLOT void collectMove(QPointF start, QPointF end, int color);

    /*! Emit the move from the notification to the rest of the UI
     *  \param
     */
    Q_SLOT void movePiece(CMove move);

protected:

    void resizeEvent(QResizeEvent *event);

public:

    /*!
     *  \param parent
     */
    explicit Qt2DDisplay(QWidget* parent=NULL);

    /*!
     */
    ~Qt2DDisplay();

    /*! \brief notification of a new move played
     *  \param move
     */
    void notify(const Move<ChessBoard::Coords> & move);

    /*! \brief create the piece corresponding to the \a position
     *  \param position
     */
    void displayPiece(const ChessBoard::Coords& position)
    {
        int pieceX = m_caseWidth * position.column;
        int pieceY = m_caseHeight * position.line;

        const Piece& piece = m_pBoard->get(position.get());
        Qt2DPiece* pItem = NULL;

        if ( !piece.isValid() )
        {
            return;
        }

        if ( piece.color() == 0 ) // White
        {
            pItem = new Qt2DPiece(m_piecesPixmap[piece.type()].first,piece.color());
        }
        else
        {
            pItem = new Qt2DPiece(m_piecesPixmap[piece.type()].second,piece.color());
        }
        connect(pItem,SIGNAL(pieceMoved(QPointF,QPointF,int)),this,SLOT(collectMove(QPointF,QPointF,int)));
        connect(pItem,SIGNAL(requestHelper(QPointF,int)),this,SLOT(displayHelper(QPointF,int)));
        connect(pItem,SIGNAL(stopHelper()),this,SLOT(stopHelper()));

        // Handling scale (since we can resize the game board
        float scaleX = m_caseWidth/pItem->boundingRect().width();
        float scaleY = m_caseHeight/pItem->boundingRect().height();
        float scale = std::min(scaleX,scaleY);

        pItem->setScale(scale);

        m_scene.addItem(pItem);

        // Position piece and get it centered
        pItem->setPos(pieceX + (m_caseWidth-(scale*pItem->boundingRect().width()))/2.0f,
                      pieceY + (m_caseHeight-(scale*pItem->boundingRect().height()))/2.0f);

        m_pieces.push_back(pItem);
    }

    void setBoard(const ChessBoard *pBoard);

    /*! \brief Set the rules to play with
     *  \param pRules
     */
    void setRules(const ChessRules* pRules) { m_pRules = pRules; }

    /*! \brief Displays helpers
     *  Helpers show which moves are available :
     *  - Green is for possible moves
     *  - Blue if for a move eating a piece
     *  - Red is for piece attacking you
     *  \param position where the click was
     *  \param color the color of the piece clicked
     */
    Q_SLOT void displayHelper(QPointF pos, int color);

    /*! \brief Stop to display helpers
     *  Removes all helpers
     */
    Q_SLOT void stopHelper();

};

#endif
