#include "GameWidget.hpp"

#include <QMenu>
#include <QMenuBar>
#include <QAction>

#include <QDebug>
#include <QMessageBox>
#include <QFileDialog>
#include <QDate>

#include "NewGameDialog.hpp"

#include "Game/AbstractGame.hpp"
#include "Game/Game.hpp"
#include "GameFactory.hpp"
#include "GameLauncher.hpp"

#include "Game/Rules/ChessRules.hpp"
#include "Controlers/QtControler.hpp"
#include "Controlers/QtGNUChessControler.hpp"
#include "Controlers/GNUChessControler.hpp"

GameWidget::GameWidget(QWidget *parent)
    :QMainWindow(parent),m_pGameThread(NULL),m_pGame(NULL),m_pRenderer(NULL)
{
    // Adds a game menu
    {
        QMenu* pGameMenu = new QMenu(tr("&Game"),this);

        QAction* pNewGameAction = new QAction(tr("&New Game"),pGameMenu);
        connect(pNewGameAction,SIGNAL(triggered()),this,SLOT(newGame()));
        pGameMenu->addAction(pNewGameAction);

        pGameMenu->addSeparator();

        QAction* pLoadAction = new QAction(tr("&Load"),pGameMenu);
        connect(pLoadAction,SIGNAL(triggered()),this,SLOT(loadGame()));
        pGameMenu->addAction(pLoadAction);

        m_pSaveAction = new QAction(tr("&Save"),pGameMenu);
        m_pSaveAction->setEnabled(false);
        connect(m_pSaveAction,SIGNAL(triggered()),this,SLOT(saveGame()));
        pGameMenu->addAction(m_pSaveAction);

        pGameMenu->addSeparator();

        QAction* pQuitAction = new QAction(tr("&Quit"),pGameMenu);
        connect(pQuitAction,SIGNAL(triggered()),this,SLOT(close()));
        pGameMenu->addAction(pQuitAction);

        this->menuBar()->addMenu(pGameMenu);
    }
}

template <typename GameType>
void GameWidget::createControlers(GameType* pGame, const std::vector<int>& controlerTypes)
{
    for ( size_t i = 0 ; i < controlerTypes.size() ; i++ )
    {
        switch(controlerTypes[i])
        {
            case GameFactory::CT_GNUChess:
            {
                QtGNUChessControler* pControler = new QtGNUChessControler();
                pControler->start();
                pGame->addControler(pControler);
                break;
            }
            case GameFactory::CT_Player:
            {
                QtControler* pControler = new QtControler();
                pGame->addControler(pControler);
                connect(m_pRenderer,SIGNAL(movePlayed(Move<ChessBoard::Coords>)),pControler,SLOT(moveReceiver(Move<ChessBoard::Coords>)));
                break;
            }
            default:
                break;
        }
    }
}

void GameWidget::createGame(int gameType, const std::vector<int>& controlerTypes)
{
    if ( GameFactory::getNbControlersNeeded(gameType) != static_cast<int>(controlerTypes.size()) )
    {
        std::cout << "Not enough controlers for this game" << std::endl;
        assert(false);
        return;
    }

    m_pRenderer = new Qt2DDisplay(this);
    this->setCentralWidget(m_pRenderer);

    AbstractGame* pAbstractGame = NULL;
    switch (gameType)
    {
        case GameFactory::GT_Chess:
        {
            Game<ChessBoard, ChessRules>* pGame = new Game<ChessBoard, ChessRules>();
            createControlers<Game<ChessBoard, ChessRules> >(pGame,controlerTypes);

            m_pRenderer->setBoard(pGame->getBoard());
            m_pRenderer->setRules(pGame->getRules());
            pGame->addStatusListener(m_pRenderer);

            pAbstractGame = pGame;
            break;
        }
        default:
            break;
    }

    m_pGame = pAbstractGame;
}

void GameWidget::destroyGame()
{
    m_pSaveAction->setEnabled(false);

    if ( m_pGameThread )
    {
        m_pGameThread->terminate();
        m_pGameThread->wait();
    }
    delete m_pGameThread;

    delete m_pRenderer;
    delete m_pGame;
}

void GameWidget::newGame()
{
    NewGameDialog ngd(this);
    if ( ngd.exec() == QDialog::Accepted )
    {
        destroyGame();

        createGame(ngd.getGameType(),ngd.getControlerTypes());
        if ( m_pGame)
        {
            m_pGameThread = new GameLauncher(m_pGame);

            connect(m_pGameThread,SIGNAL(gameDraw()),this,SLOT(draw()));
            connect(m_pGameThread,SIGNAL(gameVictory(int)),this,SLOT(victory(int)));

            m_pGameThread->start();

            this->resize(800,600);

            m_pSaveAction->setEnabled(true);
        }
    }
}

void GameWidget::loadGame()
{
    QDir dir = QDir::current();
    if ( !dir.exists("./savedGames/") )
    {
        dir.mkdir("./savedGames/");
    }

    QString fileName = QFileDialog::getOpenFileName(this,"Load game","./savedGames","*.chess");
    if ( !fileName.isEmpty() )
    {
        destroyGame();

        // Create a chess game
        std::vector<int> controlers(2,GameFactory::CT_Player);
        createGame(GameFactory::GT_Chess,controlers);

        m_pGame->loadHistory(fileName.toStdString());

        m_pGameThread = new GameLauncher(m_pGame);

        connect(m_pGameThread,SIGNAL(gameDraw()),this,SLOT(draw()));
        connect(m_pGameThread,SIGNAL(gameVictory(int)),this,SLOT(victory(int)));

        m_pGameThread->start();
    }
}

void GameWidget::saveGame()
{
    QDir dir = QDir::current();
    if ( !dir.exists("./savedGames/") )
    {
        dir.mkdir("./savedGames/");
    }

    QString fileName = QFileDialog::getSaveFileName(this,"Save game","./savedGames/" + QDate::currentDate().toString("yyyy-MM-dd") + ".chess","*.chess");
    if ( !fileName.isEmpty() )
    {
        if ( !fileName.endsWith(".chess") )
        {
            fileName.append(".chess");
        }

        m_pGame->saveHistory(fileName.toStdString());
    }
}

void GameWidget::draw()
{
    QMessageBox mBox;

    mBox.setText("Draw !");

    mBox.exec();
}

void GameWidget::victory(int color)
{
    QMessageBox mBox;
    switch (color)
    {
    case 0:
        mBox.setText("White wins the game");
        break;
    case 1:
        mBox.setText("Black wins the game");
        break;
    }
    mBox.exec();
}
